;+
; NAME:
;       GEN_EIGENPROBLEM
; PURPOSE:
;       Solve the generalized eigenproblem
;                    C##a = lambda*B##a
;       using Cholesky factorization
; AUTHOR:
;       Mort Canty (2006)
;       Juelich Research Center
;       m.canty@fz-juelich.de
; CALLING SEQUENCE:
;       Gen_Eigenproblem, C, B, A, lambda
; ARGUMENTS:
;       C and B are real, square, symmetric matrices
;       B is positive definite
;       returns the eigenvalues in the row vector lambda
;       returns the eigenvectors a in the columns of A
; KEYWORDS:
;       None
; DEPENDENCIES
;       None
;-------------------------------------------------------
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;----------------------------------------------------------

pro gen_eigenproblem, C,B,A,lambda
; solve the generalized eigenproblem C##a = lambda*B##a
   choldc, B, P, /double
   for i=1,(size(B))[1]-1 do B[i,0:i]=[fltarr(i),P[i]]
   B[0,0]=P[0]
   Li  = invert(B,/double)
   D = Li ## C ## transpose(Li)
; ensure symmetry after roundoff errors
   D = (D+transpose(D))/2
   lambda = eigenql(D,/double,eigenvectors=A)
   A = transpose(A##Li)
end